@@ -3,6 +3,15 @@
 
 #include "libsupport.h"
 #include <sys/types.h>
+#ifdef HAVE_ALLOCA_H
+# define USE_ALLOCA
+#endif
+#ifdef USE_UNISTD
+# include <unistd.h>
+#endif
+#ifdef USE_STDLIB
+# include <stdlib.h>
+#endif
 
 #ifndef	__
 # ifndef __STDC__
