@@ -119,7 +119,8 @@ extern int	       ctlsticky __((char *spec_host,char *addr_host));
 /* diagnostic.c: */
 extern char           *notaryacct __((int rc, char *okstr));
 extern void 	       notaryreport __((char*,char*,char*));
-extern void	       diagnostic __(());
+extern void	       notarystatsave __((char *smtpstatline));
+extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 
 /* emptyline.c: */
 extern int	       emptyline __((char *line, int size));
@@ -172,7 +173,5 @@ extern int writeheaders __((struct rcpt *rp, FILE *fp, char *newline, int use_cv
 extern char *mydomain __((void));
 extern char *buildboundary __((char *what));
 
-extern void diagnostic __(());
-extern void notaryreport __(());
 extern int getout;
 extern SIGNAL_TYPE wantout __(());
