@@ -18,7 +18,7 @@ mail_alloc(nbytes)
 	return (char*)malloc(nbytes);
 }
 
-int
+void
 mail_free(s)
 	char *s;
 {
