@@ -1,4 +1,3 @@
-/* XX: turned all  /stderr/ to /stdout/ 940906/mea */
 #include "hostenv.h"
 #include "mailer.h"
 #include <ctype.h>
@@ -763,17 +762,16 @@ tregexec(prog, tstring)
 
 #ifdef	DEBUG
 	if (D_compare) {
-
 		fprintf(stderr,
 			"%*scomparing %s and ", 4*funclevel, "", prog->pattern);
 		if (tstring != NULL) {
-			fputc('\'', stderr);
+			putc('\'', stderr);
 			for (s = tstring; s != NULL ; s = s->t_next)
 				fprintToken(stderr, s, 0);
-			fputc('\'', stderr);
+			putc('\'', stderr);
 		} else
 			fprintf(stderr, "(nil)");
-		fputc('\n', stderr);
+		putc('\n', stderr);
 	}
 #endif	/* DEBUG */
 
@@ -827,7 +825,6 @@ tregexec(prog, tstring)
 success:
 #ifdef	DEBUG
 	if (D_matched) {
-		
 		fprintf(stderr,
 			"%*smatched %s and '", 4*funclevel, "", prog->pattern);
 		for (s = tstring; s != NULL ; s = s->t_next)
@@ -1250,12 +1247,12 @@ tregdump(r)
 #endif
 			/* Literal string, where present. */
 			while (*s != '\0') {
-				putchar(*s);
+				putc(*s,stderr);
 				s++;
 			}
 			s++;
 		}
-		putchar('\n');
+		putc('\n',stderr);
 	}
 
 	/* Header fields of interest. */
