@@ -37,7 +37,8 @@ search_seq(sip)
 	register unsigned char *cp, *s;
 	struct conscell *tmp;
 	struct spblk *spl;
-	int retry, symid;
+	int retry;
+	spkey_t symid;
 	char buf[BUFSIZ];
 	struct file_map *fm;
 #ifdef	USE_MMAP
@@ -160,7 +161,7 @@ close_seq(sip)
 {
 	struct file_map *fm;
 	struct spblk *spl;
-	int symid;
+	spkey_t symid;
 
 	if (sip->file == NULL)
 		return;
@@ -189,7 +190,8 @@ open_seq(sip, mode)
 {
 	struct file_map *fm;
 	struct spblk *spl;
-	int symid, imode;
+	spkey_t symid;
+	int imode;
 #ifdef	USE_MMAP
 	struct stat fst;
 #endif
@@ -312,7 +314,8 @@ modp_seq(sip)
 	FILE *fp;
 	struct stat stbuf;
 	struct spblk *spl;
-	int symid, rval;
+	spkey_t symid;
+	int rval;
 
 	if (sip->file == NULL || (fp = open_seq(sip, "r")) == NULL)
 		return 0;
@@ -326,11 +329,11 @@ modp_seq(sip)
 	symid = symbol((u_char *)sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
-		rval = (stbuf.st_mtime != (time_t)spl->data
-			|| stbuf.st_nlink != (int)spl->mark);
+		rval = ((long)stbuf.st_mtime != (long)spl->data
+			|| (long)stbuf.st_nlink != (long)spl->mark);
 	} else
 		rval = 0;
-	sp_install(symid, (u_char *)stbuf.st_mtime,
+	sp_install(symid, (u_char *)((long)stbuf.st_mtime),
 		   stbuf.st_nlink, spt_modcheck);
 	return rval;
 }
@@ -391,7 +394,8 @@ readchunk(file, offset)
 	unsigned char *as;
 	struct conscell *tmp, *l;
 	struct spblk *spl;
-	int retry, symid, flag, len;
+	int retry, flag, len;
+	spkey_t symid;
 	char buf[BUFSIZ];
 	struct file_map *fm;
 #ifdef USE_MMAP
