@@ -71,6 +71,7 @@ static int vtxprep_skip_any  = 0;
 static int vtxprep_skip_lock = 0;
 static time_t next_dirscan     = 0;
 static time_t next_idlecleanup = 0;
+static struct sptree *dirscan_mesh = NULL;
 
 #include "memtypes.h"
 extern memtypes stickymem;
@@ -128,6 +129,39 @@ int global_maxkids = 1000;
 time_t now;
 FILE *statuslog = NULL;
 
+void cfp_free(cfp)
+struct ctlfile *cfp;
+{
+	struct vertex *vp, *nvp;
+	struct spblk *spl;
+
+	/* Delete from the  spt_mesh[]  */
+
+	spl = sp_lookup(cfp->id, spt_mesh[L_CTLFILE]);
+	if (spl != NULL)
+	  sp_delete(spl, spt_mesh[L_CTLFILE]);
+
+	/* And from memory */
+
+	if (cfp->head != NULL) {
+	  for (vp = cfp->head; vp != NULL; vp = nvp) {
+	    nvp = vp->next[L_CTLFILE];
+	    unvertex(vp,1); /* Don't unlink! Just free()! */
+	  }
+	} else {
+	  free(cfp->contents);
+	  free((char *)cfp);
+	}
+}
+
+static int ctl_free __((struct spblk *spl));
+static int ctl_free(spl)
+struct spblk *spl;
+{
+	cfp_free((struct ctlfile *)spl->data);
+	return 0;
+}
+
 int
 main(argc, argv)
 	int argc;
@@ -145,6 +179,7 @@ main(argc, argv)
 	/* setlinebuf(stderr);  -- no need for this ? */
 
 	memset(&dqb,0,sizeof(dqb));
+	dirscan_mesh = sp_init();
 
 	if ((progname = strrchr(argv[0], '/')) == NULL)
 		progname = argv[0];
@@ -363,11 +398,13 @@ main(argc, argv)
 
 	  if (now >= next_dirscan) {
 	    /* Directory scan time for new jobs .. */
-	    next_dirscan = now + sweepinterval;	/* 10 seconds interval */
-	    dirqueuescan(".", dq);
-	    if (dq->wrkcount > 100)
-	      next_dirscan = now + 60; /* Lots of work, pick new jobs
-					  less frequently */
+	    if (dirqueuescan(".", dq) < 150) {
+	      /* If we have more things to scan, don't quit yet! */
+	      next_dirscan = now + sweepinterval;	/* 10 seconds interval */
+	      if (dq->wrkcount > 100)
+		next_dirscan = now + 60; /* Lots of work, pick new jobs
+					    less frequently */
+	    }
 	  }
 
 	  if (now >= next_idlecleanup) {
@@ -412,13 +449,17 @@ main(argc, argv)
 	  } while (mux(timeout) == 0 && !gotalarm &&
 		   dq->wrkcount == 0 && !mustexit);
 	} while (!mustexit);
+
+	/* Doing nicely we would kill the childs here, but we are not
+	   such a nice people -- we just discard incore data, and crash out.. */
+
+	sp_scan(ctl_free, NULL, spt_mesh[L_CTLFILE]);
+
 #ifdef	MALLOC_TRACE
 	mal_dumpleaktrace(stderr);
 #endif	/* MALLOC_TRACE */
 	if (mustexit)
 		die(0, "signal");
-	exit(0);
-	/* NOTREACHED */
 	return 0;
 }
 
@@ -472,11 +513,8 @@ static int in_dirscanqueue(dq,ino)
 
 	/* Return 1, if can find the "ino" in the queue */
 
-	if (dq->wrkcount == 0) return 0;
-	for (i=0; i < dq->wrkcount; ++i)
-	  if (ino == dq->stats[i]->ino)
-	    return 1;
-
+	if (dq->wrkcount == 0 || dirscan_mesh == NULL) return 0;
+	if (sp_lookup(ino,dirscan_mesh) != NULL) return 1;
 	return 0;
 }
 
@@ -519,6 +557,9 @@ if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
 	dirp = opendir(dir);
 	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 	  /* Scan filenames into memory */
+
+	  if (newents > 200) break; /* At most 200 per one go */
+
 	  if (dp->d_name[0] >= '0' &&
 	      dp->d_name[0] <= '9') {
 	    /* A file whose name STARTS with a number (digit) */
@@ -555,6 +596,7 @@ if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
 	    memcpy(&(dsn->st),&stbuf,sizeof(stbuf));
 	    dsn->ino = ino;
 	    strcpy(dsn->name,file);
+	    sp_install(ino,(u_char*)dsn,0,dirscan_mesh);
 
 	    if (dq->wrkspace <= dq->wrkcount) {
 	      /* Increase the space */
@@ -615,6 +657,8 @@ static int syncweb(dq, cehead)
 	ino   =   dq->stats[dq->wrkcount]->ino;
 	/* Now we have pointers */
 
+	sp_delete(sp_lookup(ino,dirscan_mesh),dirscan_mesh); /* XX: should ALWAYS succeed.. */
+
 	/* Sometimes we may get files already in processing
 	   into our pre-schedule queue */
 
@@ -671,7 +715,7 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	  return NULL;
 	}
 	if (cfp->head == NULL) {
-	  unctlfile(cfp);
+	  unctlfile(cfp, 0);
 	  return NULL;
 	}
 	for (vp = cfp->head; vp != NULL; vp = vp->next[L_CTLFILE]) {
@@ -855,7 +899,7 @@ static struct ctlfile *vtxprep(cfp)
 	struct spblk *spl;
 	struct vertex *vp, *pvp, **pvpp, *head;
 	struct stat stbuf;
-	struct offsort *offarr = NULL;
+	struct offsort *offarr;
 	int offspc;
 
 	if (cfp == NULL)
@@ -863,9 +907,11 @@ static struct ctlfile *vtxprep(cfp)
 
 	sp_install(cfp->id, (u_char *)cfp, 0, spt_mesh[L_CTLFILE]);
 	channel = host = NULL;
+
 	/* copy offsets into local array */
 	offspc = 16;
 	offarr = (struct offsort *)emalloc(offspc * sizeof(struct offsort));
+
 	opcnt = 0;
 	svn = 0;
 	vtxprep_skip = 0;
@@ -885,18 +931,15 @@ static struct ctlfile *vtxprep(cfp)
 	       * won't errorneously delete it.
 	       */
 	      close(cfp->fd);	/* Was opened on  schedule() */
-	      spl = sp_lookup(cfp->id, spt_mesh[L_CTLFILE]);
-	      if (spl != NULL)
-		sp_delete(spl, spt_mesh[L_CTLFILE]);
 	      if (cfp->vfp != NULL) {
 		fprintf(cfp->vfp,
 			"New scheduler: Skipped a job-file because it is held locked by PID=%6.6s\n",cp+1);
 		fclose(cfp->vfp);
 	      }
-	      free(cfp->contents);
-	      free((char *)cfp);
+	      cfp_free(cfp);
 	      ++vtxprep_skip;
 	      ++vtxprep_skip_lock;
+	      free(offarr);
 	      return NULL;
 	    }
 	    *cp = _CFTAG_NORMAL; /* unlock it */
@@ -1051,16 +1094,13 @@ static struct ctlfile *vtxprep(cfp)
 	  sprintf(mfpath, "../%s/%s", QUEUEDIR, cfp->mid);
 	if (cfp->mid == NULL || cfp->logident == NULL
 	    || estat(mfpath, &stbuf) < 0) {
-	  spl = sp_lookup(cfp->id, spt_mesh[L_CTLFILE]);
-	  if (spl != NULL)
-	    sp_delete(spl, spt_mesh[L_CTLFILE]);
 	  if (cfp->vfp != NULL) {
 	    fprintf(cfp->vfp,
 		    "aborted due to missing information\n");
 	    fclose(cfp->vfp);
 	  }
-	  free(cfp->contents);
-	  free((char *)cfp);
+	  cfp_free(cfp);
+	  free(offarr);
 	  return NULL;
 	}
 	cfp->ctime = stbuf.st_ctime;
@@ -1210,6 +1250,7 @@ static struct ctlfile *vtxprep(cfp)
 	   }
 	*/
 	cfp->head = head;
+	free(offarr);
 	return cfp;
 }
 
