@@ -26,7 +26,7 @@ struct thread      *thread_head = NULL;
 struct thread      *thread_tail = NULL;
 
 static struct threadgroup *thrg_root   = NULL;
-static int idleprocs = 0;
+int idleprocs = 0;
 
 struct threadgroup *
 create_threadgroup(cep, wc, wh, withhost)
@@ -791,6 +791,7 @@ if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
 	  /* Move the kid to the new host web */
 	  proc->ho = thr->vertices->orig[L_HOST];
 	  proc->ho->kids += 1;
+	  thr0->proc = NULL; /* no longer in there .. */
 	  thr->proc = proc;
 	  return; /* It is eligible to run! */
 	}
@@ -799,6 +800,7 @@ if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
 	proc->thread = NULL;
 	proc->vertex = NULL;
 	proc->next = thg->idleproc;
+	thr0->proc = NULL;
 	thg->idleproc = proc;
 	thg->idlecnt += 1;
 	++idleprocs;
@@ -810,11 +812,39 @@ if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
  */
 
 void
-thread_reschedule(thr)
+thread_reschedule(thr, retrytime, index)
 struct thread *thr;
+int index;
+time_t retrytime;
 {
-	/* XXX: Don't worry right now about  thread_reschedule().
-	        Propably no need, as reschedule() does most of it (all?) */
+	struct vertex *vtx = thr->vertices;
+	time_t wakeup = 0;
+
+	time(&now);
+
+	if (verbose)
+	  printf("thread_reschedule() ch=%s ho=%s jobs=%d\n",
+		 thr->channel,thr->host,thr->jobs);
+
+	/* Time to expire ? */
+	if (vtx->ce.expiry > 0 && vtx->ce.expiry <= now && vtx->attempts > 0) {
+	  expire(vtx,index);
+	  return;
+	}
+
+	/* Reschedule ALL vertices on this thread */
+	while (vtx) {
+	  if (vtx->wakeup < retrytime)
+	    vtx->wakeup = retrytime;
+	  if (wakeup > vtx->wakeup || wakeup == 0)
+	    wakeup = vtx->wakeup;
+
+	  /* Mark it busy... */
+	  vtx->proc = thr->proc;
+
+	  vtx = vtx->nextitem;
+	}
+	thr->wakeup = wakeup;
 }
 
 
@@ -903,7 +933,8 @@ reschedule(vp, factor, index)
 	  vp->wakeup = ((((now - vp->wakeup) / vp->ce.interval)+1)
 			* vp->ce.interval);
 
-	if (vp->ce.expiry > 0 && vp->ce.expiry <= vp->wakeup
+	if (vp->ce.expiry > 0
+	    && vp->ce.expiry <= vp->wakeup
 	    && vp->attempts > 0) {
 	  if (verbose)
 	    printf("ce.expiry = %d, %d attempts\n",
@@ -954,7 +985,6 @@ reschedule(vp, factor, index)
 	else
 	  SALARM(sweepinterval/2);
 #endif
-	thread_reschedule(thr);
 }
 
 
@@ -1039,6 +1069,7 @@ FILE *fp;
 {
 	struct threadgroup *thg;
 	int thg_once = 1;
+	int jobsum = 0, jobtotal = 0;
 
 	if (thrg_root == NULL) {
 	  fprintf(fp,"No threads/processes.\n");
@@ -1060,6 +1091,7 @@ FILE *fp;
 		  thg->cep->channel, thg->cep->host, thg->withhost);
 	  cnt   = 0;
 	  procs = 0;
+	  jobsum = 0;
 	  thr_once = 1;
 	  for (thr = thg->thread; thr && (thr_once || thr != thg->thread); thr = thr->nextthg) {
 	    thr_once = 0;
@@ -1068,6 +1100,7 @@ FILE *fp;
 		    /* thr->vertices->orig[L_CHANNEL]->name, */ thr->channel,
 		    /* thr->vertices->orig[L_HOST]->name, */ thr->host,
 		    thr->thgrp->withhost,thr->jobs);
+	    jobsum += thr->jobs;
 	    ++cnt;
 	    if (thr->proc != NULL &&
 		thr->proc->thread == thr) {
@@ -1084,9 +1117,12 @@ FILE *fp;
 	  }
 	  fprintf(fp,"\t\tThreads: %d/%d ",thg->threads,cnt);
 	  fprintf(fp,"Procs: %d/%d ",thg->transporters,procs);
+	  fprintf(fp,"Jobs: %d ",jobsum);
+	  jobtotal += jobsum;
 	  cnt = 0;
 	  for (;p != 0; p = p->next) ++cnt;
 	  fprintf(fp,"Idle-procs: %d/%d\n",thg->idlecnt,cnt);
 	}
-	fprintf(fp,"Idleprocs: %d   NumKids: %d\n",idleprocs, numkids);
+	fprintf(fp,"NumKids: %d   Idleprocs: %d   Jobs: %d\n",
+		numkids, idleprocs, jobtotal);
 }
