@@ -348,14 +348,20 @@ if (verbose)
 	      *pp = p->next;
 	      p = p->next;
 	      proc->thg->idlecnt      -= 1;
+	      --idleprocs;
 	      proc->thg->transporters -= 1;
 	      --numkids;
 	      /* It may go down to zero.. */
 	      if (proc->thg->transporters == 0 && proc->thg->threads == 0)
 		delete_threadgroup(proc->thg);
 	    } else {
-	      fprintf(stderr, "reclaim(fd=%d) proc->thg=0x%lx, proc=0x%lx NOT IN IDLE CHAIN!\n", fromfd, (long)proc->thg, (long)proc);
-	      proc->thg->idlecnt      -= 1;
+	      /* It is not in idle chain, it has died somehow else.. */
+	      if (proc->vertex != NULL)
+		proc->vertex->proc = NULL;
+	      proc->vertex = NULL;
+	      if (proc->thread != NULL)
+		proc->thread->proc = NULL;
+	      proc->thread = NULL;
 	      proc->thg->transporters -= 1;
 	      --numkids;
 	    }
@@ -802,17 +808,18 @@ int signum;
 #ifdef	WIFEXITED
 	  if (WIFEXITED(statloc)) ok = 1;
 #else
-      ERROR:ERROR:ERROR
+      ERROR_ERROR_ERROR:
 #endif
 #ifdef WIFSIGNALED
 	if (WIFSIGNALED(statloc)) ok = 1;
 #else
-      ERROR:ERROR:ERROR
+      ERROR_ERROR_ERROR:
 #endif
 
-	if (verbose)
-	  fprintf(stderr,"sig_chld() pid=%d, ok=%d\n",pid,ok);
-
+	  if (verbose)
+	    fprintf(stderr,"sig_chld() pid=%d, ok=%d, stat=0x%x\n",
+		    pid,ok,statloc);
+	  
 	  if (ok && cpids != NULL) {
 	    /* Only EXIT and SIGxx DEATHS accepted */
 
@@ -832,7 +839,6 @@ int signum;
 	      }
 	    }
 	  }
-
 	}
 
 	/* re-instantiate the signal handler.. */
