@@ -373,7 +373,7 @@ deletemsg(msgid, curcfp)
  */
 char *
 saytime(value, buf, shortform)
-	u_long value;
+	long value;
 	char *buf;
 	int shortform;
 {
@@ -382,6 +382,12 @@ saytime(value, buf, shortform)
 
 	p = buf;
 
+	while (*p) ++p;
+	if (value < 0) {
+	  *p++ = '-'; *p = 0;
+	  value = -value;
+	}
+
 	secs = value % 60;
 	value /= 60;
 	mins = value % 60;
@@ -390,7 +396,6 @@ saytime(value, buf, shortform)
 	value /= 24;
 
 #define	PLURALIZE(x)	x, (x == 1) ? "" : "s"
-	while (*p) ++p;
 	if (value) {
 	  if (shortform)
 	    sprintf(p, "%ldd", value);
@@ -668,6 +673,6 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	 * the heuristic in reschedule() to ignore the request if
 	 * the time is already in the future should help out.
 	 */
-	reschedule(vp, -(int)retrytime, index);
+	thread_reschedule(vp->thread, retrytime, index);
 	return 1;
 }
