@@ -32,11 +32,12 @@ FILE *qpfp;
 char qpch;
 time_t qpnow;
 
+static int qid = 0;
+
 void
 qprint(fd)
 	int fd;
 {
-	time_t now;
 #ifdef	USE_TIMEVAL
 	struct timeval tvp[2];
 #else	/* !USE_TIMEVAL */
@@ -65,7 +66,7 @@ qprint(fd)
 	}
 	qpch = '\0';
 	fprintf(qpfp, "version zmailer 1.0\nVertices:\n");
-	qpnow = time((time_t *)0);
+	time(&qpnow);
 	sp_scan(qpctlfile, (struct spblk *)NULL, spt_mesh[L_CTLFILE]);
 	if (qpch != '\0') {
 		fprintf(qpfp, "Channels:\n");
@@ -75,15 +76,17 @@ qprint(fd)
 	}
 	fclose(qpfp);
 	/* XX: I suppose we don't really need to do this if we use TCP. */
-	now = time((time_t *)0) - 1;
+	if (rendezvous != NULL) {
+	  time_t now = time((time_t *)0) - 1;
 #ifdef	USE_TIMEVAL
-	tvp[0].tv_sec = tvp[1].tv_sec = now;
-	tvp[0].tv_usec = tvp[1].tv_usec = 0;
-	utimes(rendezvous, tvp);
+	  tvp[0].tv_sec = tvp[1].tv_sec = now;
+	  tvp[0].tv_usec = tvp[1].tv_usec = 0;
+	  utimes(rendezvous, tvp);
 #else	/* !USE_TIMEVAL */
-	tvp.actime = tvp.modtime = now;
-	utime(rendezvous, &tvp);
+	  tvp.actime = tvp.modtime = now;
+	  utime(rendezvous, &tvp);
 #endif	/* USE_TIMEVAL */
+	}
 }
 
 int
@@ -97,8 +100,9 @@ qpctlfile(spl)
 
 	/* assert cfp != NULL */
 	for (vp = cfp->head; vp != NULL; vp = vp->next[L_CTLFILE]) {
-		fprintf(qpfp, "%lu:\t%s\t%d;",
-			(u_long)vp, cfp->mid, vp->ngroup);
+		vp->qid = ++qid;
+		fprintf(qpfp, "%d:\t%s\t%d;",
+			vp->qid, cfp->mid, vp->ngroup);
 		qpch = ' ';
 		for (i = 0; i < vp->ngroup; ++i) {
 			fprintf(qpfp, "%c%ld",
@@ -106,36 +110,37 @@ qpctlfile(spl)
 			qpch = ',';
 		}
 		if (vp->message != NULL)
-			fprintf(qpfp, "\t#%s", vp->message);
-		if (vp->wakeup > qpnow || vp->ce.pending) {
-			if (vp->message == NULL)
-				fprintf(qpfp, "\t#");
-			else
-				putc(' ', qpfp);
-			i = 0;
-			if (vp->wakeup > qpnow) {
-				buf[0] = '\0';
-				saytime((u_long)(vp->wakeup - qpnow), buf, 1);
-				fprintf(qpfp, "(retry in %s", buf);
-				++i;
-			} else /* if (vp->ce.pending) */ {
-				fprintf(qpfp, "(waiting for %sslot",
-					vp->ce.pending == SIZE_L ? "" :
-					vp->ce.pending == L_CHANNEL ? "channel " :
-					"host ");
-				++i;
-			}
-                        if (vp->ce.expiry > 0) {
-				/* [mea] Want to know when it expires.. */
-				buf[0] = '\0';
-				saytime((u_long)(vp->ce.expiry - qpnow),
-					buf, 1);
-				fprintf(qpfp,"%sexpires in %s",
-					i ? ", " : "(", buf);
-                        }
-			if (i)
-				putc(')', qpfp);
+			fprintf(qpfp, "\t#%s ", vp->message);
+		else
+			fprintf(qpfp, "\t#");
+
+		i = 0;
+		if (vp->wakeup > qpnow) {
+			buf[0] = '\0';
+			saytime((u_long)(vp->wakeup - qpnow), buf, 1);
+			fprintf(qpfp, "(retry in %s", buf);
+			++i;
+		} else if (vp->ce.pending) {
+			fprintf(qpfp, "(waiting for %sslot",
+				vp->ce.pending == SIZE_L ? "" :
+				vp->ce.pending == L_CHANNEL ? "channel " :
+				"host ");
+			++i;
+		} else {
+			fprintf(qpfp, "(running now");
+			++i;
 		}
+		if (vp->ce.expiry > 0) {
+			/* [mea] Want to know when it expires.. */
+			buf[0] = '\0';
+			saytime((u_long)(vp->ce.expiry - qpnow),
+				buf, 1);
+			fprintf(qpfp,"%sexpires in %s",
+				i ? ", " : "(", buf);
+		}
+		if (i)
+			putc(')', qpfp);
+
 		putc('\n', qpfp);
 	}
 	return 0;
@@ -150,7 +155,7 @@ qpchannel(spl)
 
 	(void) fprintf(qpfp, "%s:\t", wc->name);
 	for (vp = wc->link; vp != NULL; vp = vp->next[L_CHANNEL])
-		fprintf(qpfp, ">%lu", (u_long)vp);
+		fprintf(qpfp, ">%d", vp->qid);
 	fprintf(qpfp, "\n");
 	return 0;
 }
@@ -164,7 +169,7 @@ qphost(spl)
 
 	fprintf(qpfp, "%s:\t", wc->name);
 	for (vp = wc->link; vp != NULL; vp = vp->next[L_HOST])
-		fprintf(qpfp, ">%lu", (u_long)vp);
+		fprintf(qpfp, ">%d", vp->qid);
 	fprintf(qpfp, "\n");
 	return 0;
 }
