@@ -2,6 +2,10 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Several extensive changes by Matti Aarnio <mea@nic.funet.fi>
+ *	Copyright 1991-1995.
+ */
 
 /*
  * ZMailer SMTP server.
@@ -9,7 +13,7 @@
 
 char *VerbID = "ZMailer SMTP server %s";
 char *Copyright = "Copyright 1990 Rayan S. Zachariassen";
-char *Copyright2= "Copyright 1991-1994 Matti Aarnio";
+char *Copyright2= "Copyright 1991-1995 Matti Aarnio";
 
 /* Timing parameters -- when expired, session is killed ... */
 
@@ -146,7 +150,8 @@ struct smtpconf {
 };
 
 extern char *rfc821_domain __((char *s, int strict));
-extern char *rfc821_path __((char *s, int strict));
+extern char *rfc821_path   __((char *s, int strict));
+extern char *rfc821_path2  __((char *s, int strict));
 extern char *rfc821_error;
 extern char *rfc821_error_ptr;
 extern void type821err();
@@ -1485,11 +1490,10 @@ smtpserver(insecure)
 	    case Verify2:
 		if ((cfi = STYLE(cfinfo, 'v'))) {
 		  while (*cp == ' ' || *cp == '\t') ++cp;
-		  if (*cp != '<') {
-		    type(501, "Growl!  Expecting arguments in RFC-821 format:  VRFY <some@address>");
-		    break;
-		  }
-		  s = rfc821_path(cp,cfi);
+		  if (*cp == '<')
+		    s = rfc821_path(cp,cfi); /* with < > */
+		  else
+		    s = rfc821_path2(cp,cfi); /* Without < > */
 		  if (s == cp) {
 		    type821err(501,buf,"Path data: %s",rfc821_error);
 		    break;
@@ -1509,11 +1513,10 @@ smtpserver(insecure)
 	    case Expand:
 		if ((cfi = STYLE(cfinfo, 'e'))) {
 		  while (*cp == ' ' || *cp == '\t') ++cp;
-		  if (*cp != '<') {
-		    type(501, "Growl!  Expecting arguments in RFC-821 format:  EXPN <some@address>");
-		    break;
-		  }
-		  s = rfc821_path(cp,cfi);
+		  if (*cp == '<')
+		    s = rfc821_path(cp,cfi); /* with < > */
+		  else
+		    s = rfc821_path2(cp,cfi); /* Without < > */
 		  if (s == cp) {
 		    type821err(501,buf,"Path data: %s",rfc821_error);
 		    break;
@@ -1800,7 +1803,7 @@ mgets(buf, flagp, fp)
 	int *flagp;
 	FILE *fp;
 {
-	register int bufsize;
+	register int bufsize, c;
 	register char *cp;
 	char *s;
 	static char *obuf = NULL;
@@ -1814,8 +1817,9 @@ mgets(buf, flagp, fp)
 
 	bufsize = 16;
 	cp = buf = emalloc(bufsize);
-	while ((*cp = getc(fp)) != EOF) {
-	  if (*cp == '\n') {
+	while ((c = getc(fp)) != EOF) {
+	  *cp = c;
+	  if (c == '\n') {
 	    *flagp = 0;
 	    break;
 	  }
@@ -1832,7 +1836,7 @@ mgets(buf, flagp, fp)
 	    buf = s;
 	  }
 	}
-	if (*cp == EOF) {
+	if (c == EOF) {
 	  printf("got EOF!\n");
 	  free(buf);
 	  return NULL;
@@ -2139,6 +2143,13 @@ type821err(code,inbuf,msg,s1,s2,s3)
 	type(code,msg,s1,s2,s3);
 }
 
+static void header_to_mime(buf, lenptr, maxlen)
+	char *buf;
+	int *lenptr;
+	int maxlen;
+{
+	/* XXX: HEADERS -> MIME-2 */
+}
 
 /* Implement SMTP DATA filter */
 
@@ -2225,7 +2236,7 @@ mvdata(out,msg)
 	sts = states;
 	cnt = 0;
 #else
-	char linebuf[2000], *s, *eol;
+	char linebuf[4000], *s, *eol;
 	int col;
 	int insubject = 0;
 	int has8bit = 0; /* In headers */
@@ -2263,80 +2274,111 @@ mvdata(out,msg)
 		  timercnt = 1024;
 		}
 		++cnt;
-		if (0x80 & c) {
-		  if (!was8bit) putc(' ',out);
-		  fprintf(out,"=?UNKNOWN-8BIT?Q?=%02X?= ", c & 0xFF);
-		  was8bit = 1;
-		} else {
-		  was8bit = 0;
-		  if ((state = sts[state+idxnum[c]]) & ~0xff) {
-		    if (state & O_) {
-		      if (state == endstate)
-			return cnt;
-		      state = (char)state;
-		      continue;
+		if ((state = sts[state+idxnum[c]]) & ~0xff) {
+		  if (state & O_) {
+		    if (state == endstate) {
+		      if (col > 0) {
+			if (has8bit)
+			  header_to_mime(linebuf,&col,sizeof(linebuf));
+			fwrite(linebuf,1,col,out);
+		      }
+		      return cnt;
 		    }
-		    if (!ferror(out))
-		      putc((state>>8), out);
-		    if (col < (sizeof(linebuf)-1))
-		      linebuf[col++] = (state >> 8);
 		    state = (char)state;
+		    continue;
 		  }
-		  if (!ferror(out))
-		    putc(c, out);
+		  if (col < (sizeof(linebuf)-1))
+		    linebuf[col++] = (state >> 8);
+		  state = (char)state;
 		}
+		if (0x80 & c)
+		  has8bit = 1;
+
 		if (col < (sizeof(linebuf)-1))
-			linebuf[col++] = c;
+		  linebuf[col++] = c;
+
 		/* LF, or something else ?  Here the else.. */
 		if (c != '\n')
-			continue;
+		  continue;
+
+		if (col < sizeof(linebuf))
+		  linebuf[col] = 0;
+
+		/* We have a string - header line - in ``col'' first
+		   char positions of the ``linebuf'' array  */
 
 		/* See what this header is about -- or if the body starts.. */
 		if (col > sizeof(linebuf)-1) col = sizeof(linebuf)-1;
+
 		eol = linebuf + col;
 		*eol = 0;
+
+		/* See if the line is all white-space: */
 		for (s = linebuf; s < eol; ++s)
-			if (*s != ' ' && *s != '\t' &&
-			    *s != '\r' && *s != '\n')
-				break;
-		if (s == eol) { /* All-blank line */
-			break;  /* Into the body processing */
+		  if (*s != ' ' && *s != '\t' &&
+		      *s != '\r' && *s != '\n')
+		    break;
+		if (s == eol) {	/* All-blank line */
+		  if (col > 0)
+		    fwrite(linebuf,1,col,out);
+		  break;	/* Into the body processing */
 		}
 		++linecnt;
-		col = 0;
-		if (*linebuf == ' ' || *linebuf == '\t')
-			continue;	/* continuation line.. */
+		if (*linebuf == ' ' || *linebuf == '\t') {
+		  if (has8bit)
+		    header_to_mime(linebuf,&col,sizeof(linebuf));
+		  has8bit = 0;
+		  if (col > 0)
+		    fwrite(linebuf,1,col,out);
+		  col = 0;
+		  continue;	/* continuation line.. */
+		}
 
 		/* ================ PROCESS THE HEADERS! ================ */
 		if (CISTREQN(linebuf,"Subject:",8)) {
-			insubject = 1;
+		  insubject = 1;
 		} else {
-			if (!insubject) {
-			  if (has8bit)
-			    sprintf(msg,"Header line \"%s\" contains illegal 8-bit chars", linebuf);
-			  has8bitsum += has8bit;
-			}
+		  if (!insubject) {
+#if 0
+		    if (has8bit)
+		      sprintf(msg,"Header line \"%s\" contains illegal 8-bit chars", linebuf);
+#endif
+		    has8bitsum += has8bit;
+		  }
 		}
 		if (linecnt == 1 && (strncmp("From ",linebuf,5)==0 ||
 				     strncmp(">From ",linebuf,6)==0)) {
+#if 0
 		  from__err = 1;
 		  sprintf(msg,"Message starts with illegal \"%s\" line",linebuf);
+#endif
+		  /* DO NOT WRITE THIS LINE OUT! */
+		  col = 0;
+		  has8bit = 0;
+		  continue;
 		}
+		if (has8bit)
+		  header_to_mime(linebuf,&col,sizeof(linebuf));
+		has8bit = 0;
+		/* Write the line out */
+		if (col > 0)
+		  fwrite(linebuf,1,col,out);
+		col = 0;
+
 		has8bit = 0;
 		if (CISTREQN(linebuf,"Precedence:",11)) {
-			s = linebuf+11;
-			while (*s == ' ' || *s == '\t') ++s;
-			if ((eol - s) < 4) continue; /* Hmm.. */
-			if (CISTREQN("junk",s,4)) {
-				mail_priority = _MAILPRIO_JUNK;
-			} else if (CISTREQN("bulk",s,5)) {
-				mail_priority = _MAILPRIO_BULK;
-			}
+		  s = linebuf+11;
+		  while (*s == ' ' || *s == '\t') ++s;
+		  if ((eol - s) < 4) continue; /* Hmm.. */
+		  if (CISTREQN("junk",s,4)) {
+		    mail_priority = _MAILPRIO_JUNK;
+		  } else if (CISTREQN("bulk",s,5)) {
+		    mail_priority = _MAILPRIO_BULK;
+		  }
 		}
 	}
-	/* if(logfp)
-	   fprintf(logfp,"%d#\t(mail_priority=%d)\n",pid,mail_priority);
-	 */
+	if (logfp)
+	  fprintf(logfp,"%d#\t(mail_priority=%d)\n",pid,mail_priority);
 #endif
 
 	/* ================ Normal email BODY input.. ================ */
