@@ -259,7 +259,7 @@ process(dp)
 	      break;
 	    }
 	    notaryreport(rp->addr->user,action,status);
-	    diagnostic(rp, rp->status, "%s", rp->addr->user);
+	    diagnostic(rp, rp->status, 0, "%s", rp->addr->user);
 	  }
 	}
 
@@ -278,7 +278,7 @@ process(dp)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,"delayed",
 			   "400 (Cannot resubmit anything, out of spool space?)");
-	      diagnostic(rp, EX_TEMPFAIL,
+	      diagnostic(rp, EX_TEMPFAIL, 0,
 			 "cannot resubmit anything!");
 	    }
 #ifdef	USE_SETREUID
@@ -347,7 +347,7 @@ process(dp)
 	  if (rp->status == EX_OK) {
 	    notaryreport(rp->addr->user,"relayed",
 			 "651 (Relayed via deferral channel)");
-	    diagnostic(rp, code, cp);
+	    diagnostic(rp, code, 0, cp);
 	  }
 }
 
