@@ -75,9 +75,9 @@ notaryreport(arg1,arg2,arg3)
 
 /*VARARGS*/
 void
-diagnostic(rp, rc, fmt, va_alist) /* (rp, rc, "fmtstr", remotemsg) */
+diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remotemsg) */
 	struct rcpt *rp;
-	int rc;
+	int rc, timeout;
 	char *fmt;
 	va_dcl
 {
@@ -133,7 +133,10 @@ diagnostic(rp, rc, fmt, va_alist) /* (rp, rc, "fmtstr", remotemsg) */
 	case EX_OSERR:
 	case EX_CANTCREAT:
 	case EX_SOFTWARE:
-		printf("deferred");
+		if (timeout > 0)
+		  printf("retryat +%d",timeout);
+		else
+		  printf("deferred");
 		mark = _CFTAG_DEFER;
 		break;
 	case EX_NOPERM:
