@@ -170,13 +170,13 @@ char *maildirs[] = {
 					(*((R)->addr->user) == TO_PIPE \
 					 && (R)->status != EX_OK \
 					 && (R)->status != EX_TEMPFAIL) ? \
-					      EX_UNAVAILABLE : (E), (A1), (A2))
+					      EX_UNAVAILABLE : (E), 0, (A1), (A2))
 
 #define	DIAGNOSTIC3(R,E,A1,A2,A3)   diagnostic((R), \
 					(*((R)->addr->user) == TO_PIPE \
 					 && (R)->status != EX_OK \
 					 && (R)->status != EX_TEMPFAIL) ? \
-					      EX_UNAVAILABLE : (E), \
+					      EX_UNAVAILABLE : (E), 0, \
 						(A1), (A2), (A3))
 
 #if	defined(BIFF) || defined(RBIFF)
