@@ -239,6 +239,7 @@ extern char *strrchr();
 extern char *dottedquad();
 extern char *optarg;
 extern int optind;
+
 extern int deliver __((struct ctldesc *dp, FILE *smtpfp[2], FILE *verboselog, struct rcpt *startrp, struct rcpt *endrp));
 extern int writebuf __((FILE *fp, char *buf, int len));
 extern int writemimeline __((FILE *fp, char *buf, int len, int convertmode, int*lastchp));
@@ -248,15 +249,8 @@ extern int smtpconn __((char *host, FILE *fparr[], FILE *verboselog));
 extern int smtp_ehlo __((FILE *smtpfp[], FILE *verboselog, char *strbuf));
 extern int ehlo_check __((char *buf));
 extern int smtpwrite __((FILE *smtpfp[2], FILE *verboselog, int saverpt, char *linebuf));
-extern int emptyline();
 extern int process __((struct ctldesc*,int,char*,FILE**,FILE*));
-extern void diagnostic();
-extern void notaryreport __((char *rcpt, char *acct, char *status));
-extern void notarystatsave __((char *smtpstatline));
-extern char *notaryacct __((int rc, char *ok_string));
 
-extern int  cte_check __((struct rcpt *rp));
-extern void downgrade_headers __((struct rcpt *rp, int convertmode, FILE *verboselog));
 extern int check_7bit_cleanness __((struct ctldesc *dp));
 
 extern void prversion();
@@ -606,7 +600,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	    } else {
 	      while (rphead != rp->next) {
 		notaryreport(rp->addr->user,"failed",NULL);
-		diagnostic(rphead, smtpstatus, "%s", remotemsg);
+		diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
 		rphead = rphead->next;
 	      }
 	    }
@@ -724,7 +718,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	if (r != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),NULL);
-	    diagnostic(rp, r, "%s", remotemsg);
+	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
 	  if (smtp_isopen)
 	    smtpwrite(smtpfp, verboselog, 0, "RSET");
@@ -760,7 +754,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	  if (r != EX_OK) {
 	    notaryreport(NULL, notaryacct(r,"*NOTOK"),NULL);
-	    diagnostic(rp, r, "%s", remotemsg);
+	    diagnostic(rp, r, 0, "%s", remotemsg);
 	    if (!smtp_isopen)
 	      break;
 	  } else {
@@ -781,7 +775,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,notaryacct(r,"*NOTOK"),NULL);
-	      diagnostic(rp, EX_TEMPFAIL, "%s", remotemsg);
+	      diagnostic(rp, EX_TEMPFAIL, 0, "%s", remotemsg);
 	    }
 	  if (smtp_isopen)
 	    smtpwrite(smtpfp, verboselog, 0, "RSET");
@@ -808,7 +802,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,notaryacct(EX_TEMPFAIL,"*NOTOK*"),
 			   "466 (Message write timed out)"); /* XX: FIX THE STATUS? */
-	      diagnostic(rp, EX_TEMPFAIL, "%s", "header write error");
+	      diagnostic(rp, EX_TEMPFAIL, 0, "%s", "header write error");
 	    }
 	  if (verboselog)
 	    fprintf(verboselog,"Writing headers after DATA failed\n");
@@ -830,7 +824,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user, notaryacct(r,"*NOTOK*"),
 			   "466 (Message write timed out)"); /* XX: FIX THE STATUS? */
-	      diagnostic(rp, r, "%s", remotemsg);
+	      diagnostic(rp, r, 0, "%s", remotemsg);
 	    }
 	  /* The failure occurred during processing and was due to an I/O
 	   * error.  The safe thing to do is to just abort processing.
@@ -872,7 +866,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    notaryreport(rp->addr->user,
 			 notaryacct(r,"relayed"), /* XXX: NOTARY DRPT ?? */
 			 NULL);
-	    diagnostic(rp, r, "%s", remotemsg);
+	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
 	if (r != EX_OK && smtp_isopen)
 	  smtpwrite(smtpfp, verboselog, 0, "RSET");
